! ************************************  main ab06_gm_fmrvs
      Program ab06_gm_fmrvs
      
* Computes AB06 motions 

*! Control file for program ab06_gm_fmrvs.for
*! Revision of program involving a change in the control file on this date:
*   02/12/10
*!Header to add to output file (no "!" at beginning!)
* [blank]
*!name of output file:
* ab06_gm_fmrvs_bc_m_6.0.out
*! Use hard rock table (Y/N)? This is included here rather than using the value of V30 to
*! determine which table to use because I do not want to open a table of coefficients
*! for each value of V30 read below.  Note that if this is "Y", the hard rock
*! table is used and the Vs30 values read from the control file are not used (in fact, the
*! input value of V30 is changed to "2500").  The downside is that the program cannot 
*! evaluate both "soil" and hard rock motions in the same run.   
* N
*!h for conversion from Rjb to Rcd; the equations are in terms of Rcd; both Rjb and Rcd are 
*! in the output file.
* 5
*!   freq     M     Rjb  Vs30 (no site amp correction if V30>= 2000.0) Stress (140 bars is the nominal stress for AB06)
*! The frequencies should fall within the 0.2 Hz to 40 Hz range of frequencies for which the AB06 
*! regression coefficients are tabulated (the program interpolates ground motions from the tabulated
*! frequencies to obtain the motion for the input frequency).  The exceptions are for PGV (freq = -1.0)
*! and PGA (freq = 99.0)
*!  f     M  Rjb   Vs30 Stress
*0.2    6.0 10.0  760.0  140.0 	 
*0.25   6.0 10.0  760.0  140.0	
*0.32   6.0 10.0  760.0  140.0	
*0.4    6.0 10.0  760.0  140.0	
*0.5    6.0 10.0  760.0  140.0	
*0.63   6.0 10.0  760.0  140.0	
*0.8    6.0 10.0  760.0  140.0	
*1      6.0 10.0  760.0  140.0	
*1.26   6.0 10.0  760.0  140.0	
*1.59   6.0 10.0  760.0  140.0	
*2      6.0 10.0  760.0  140.0	
*2.52   6.0 10.0  760.0  140.0	
*3.17   6.0 10.0  760.0  140.0	
*3.99   6.0 10.0  760.0  140.0	
*5.03   6.0 10.0  760.0  140.0	
*6.33   6.0 10.0  760.0  140.0	
*7.97   6.0 10.0  760.0  140.0	
*10.04  6.0 10.0  760.0  140.0	
*12.64  6.0 10.0  760.0  140.0	
*15.91  6.0 10.0  760.0  140.0	
*20.04  6.0 10.0  760.0  140.0	
*25.23  6.0 10.0  760.0  140.0	
*31.77  6.0 10.0  760.0  140.0	
*40     6.0 10.0  760.0  140.0
* stop	          
  
* Dates: 11/12/06 - Written by D.M. Boore, based on ab06_gm_vs_r.
*        02/05/10 - Find PSA for nontabulated frequencies by interpolating
*                   log PSA for frequencies on either side of the specified
*                   frequency.
*        02/06/10 - Added computation for other stresses (and include Vs30 in the
*                   control file, for a future modification that includes site amp).
*                   Renamed the program to reflect the input parameters.
*        02/10/10 - Removed control file option to use hard rock table.  Now base
*                   which table to use on the value of Vs30: < 2000, BC table;
*                   >= 2000, rock table.
*        02/12/10 - Removed option to use the published tables of coefficients
*                   (although I have left that option in the source code---I "hardwire"
*                   use_published_table = 'N' to disable that option)
  
  
       
      character buf*80, buf4*4, f_ctl*80, f_file*160, 
     :          f_out*160,
     :          header4output*160, format_out*160 
     
      character coeff_head(10)*3
      
      character cmnts2skip(200)*79
      
      character f_file_tbl*80

      character y_type*10,
     :          use_published_table*10, use_hard_rock_table*10 

      character control_date_correct*30, control_date_in*30

      logical file_exist 

      real m, logy_jb, mstart, mstop, dmag, 
     :     rjb, rcd, v30, stress
     
      real saab05, sigab05, freq
      
      real stress_adj_factor_delta(50)
      real stress_adj_factor_ml(50)
      real stress_adj_factor_mh(50)
      
      real freq_tbl(50), coeff(50,10) 
 
      integer nmag
  
      pi = 4.0*atan(1.0)
      twopi = 2.0 * pi

      control_date_correct = ' '
      control_date_correct = '02/12/10'
      call trim_c(control_date_correct,nc_control_date_correct)

      file_exist = .false.
      do while (.not. file_exist)
        f_ctl = ' '
        write(*, '(a\)') 
     :    ' Enter name of control file (cr = ab06_gm_fmrvs.ctl): '
        read(*, '(a)') f_ctl
        if (f_ctl .eq. ' ') f_ctl = 'ab06_gm_fmrvs.ctl'
        call trim_c(f_ctl,nc_f_ctl)
        inquire(file=f_ctl(1:nc_f_ctl), exist=file_exist)
        if (.not. file_exist) then
          write(*,'(a)') ' ******* FILE DOES NOT EXIST ******* '
        end if
      end do
      
      call get_lun(nu_ctl)
      open(unit=nu_ctl,file=f_ctl(1:nc_f_ctl),status='unknown')

      call skipcmnt(nu_ctl, cmnts2skip, nc_cmnts2skip)
      control_date_in = ' '
      read(nu_ctl,'(a)') control_date_in
      call trim_c(control_date_in,nc_control_date_in)
      
      if (control_date_correct(1:nc_control_date_correct) /=
     :    control_date_in(1:nc_control_date_in)) then
        write(*,*) ' The date in the control file is '//
     :             control_date_in(1:nc_control_date_in)
        write(*,*) ' but it should be '//
     :             control_date_correct(1:nc_control_date_correct)
        write(*,'(a)') 
     :     ' *** QUIT *** !'
        close(nu_ctl)
        stop
      end if
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)  
      header4output = ' '
      read(nu_ctl,'(a)') header4output
      call trim_c(header4output,nc_h4out)
      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)      
      f_out = ' '
      read(nu_ctl,'(a)') f_out 
      call trim_c(f_out,nc_f_out)
      call get_lun(nu_out)
      open(unit=nu_out,file=f_out(1:nc_f_out),status='unknown')

!      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)  
!      use_published_table = ' '
!      read(nu_ctl,'(a)') use_published_table
!      call upstr(use_published_table)
!      call trim_c(use_published_table,nc_use_published_table)

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)  
      use_hard_rock_table = ' '
      read(nu_ctl,'(a)') use_hard_rock_table
      call upstr(use_hard_rock_table)
      call trim_c(use_hard_rock_table,nc_use_hard_rock_table)

* READ COEFFICIENTS FROM TABLE

      use_published_table = 'N'

      call get_lun(nu_tbl)
      f_file_tbl = ' '
      IF (use_hard_rock_table(1:1) == 'Y') THEN
        IF (use_published_table(1:1) == 'Y') THEN
          f_file_tbl = '\gm_predictions\ab06acoef.txt'
        ELSE
          f_file_tbl = '\gm_predictions\AB05eqn_Rcd.par'
        END IF
      ELSE
        if (use_published_table(1:1) == 'Y') THEN
          f_file_tbl = '\gm_predictions\ab06bccoef.txt'
        ELSE
          f_file_tbl = '\gm_predictions\AB05eqnBC_Rcd.par'
        END IF
      END IF

      call trim_c(f_file_tbl,nc_f_file_tbl)
      open (nu_tbl, file=f_file_tbl(1:nc_f_file_tbl), 
     :      status='unknown')
     
      call skip(nu_tbl, 2)

      nfreq = 26
      do i = 1, nfreq
        read(nu_tbl,*) freq_tbl(i), (coeff(i,j), j = 1, 10)
      end do

      close(nu_tbl)

      write(*,*) ' Used table of coefficients: '//
     :                    f_file_tbl(1:nc_f_file_tbl)
      write(nu_out,'(a)') ' Used table of coefficients: '//
     :                    f_file_tbl(1:nc_f_file_tbl)

* Read stress adjustment factors from table

      call get_lun(nu_tbl)
      f_file_tbl = ' '
      f_file_tbl = '\gm_predictions\ab06_table_7.txt'
      call trim_c(f_file_tbl,nc_f_file_tbl)
      open (nu_tbl, file=f_file_tbl(1:nc_f_file_tbl), 
     :      status='unknown')
     
      call skip(nu_tbl, 2)

      nfreq = 26
      do i = 1, nfreq
        read(nu_tbl,*) 
     :         dum,
     :         stress_adj_factor_delta(i), 
     :         stress_adj_factor_ml(i), 
     :         stress_adj_factor_mh(i)
      end do

      close(nu_tbl)

      write(*,*) ' Used stress adjustment table: '//
     :                    f_file_tbl(1:nc_f_file_tbl)
      write(nu_out,'(a)') ' Used stress adjustment table: '//
     :                    f_file_tbl(1:nc_f_file_tbl)

!      call get_indices(freq_tbl, nfreq, 
!     :             indx_freqmax, indx_pgv, indx_pga)

* Assume coefficient table is order with increasing freq and 
* that pga and pgv correspond to the last two entries, respectivel.

      indx_freqmax = nfreq-2
      indx_pga     = nfreq-1
      indx_pgv     = nfreq


      
      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)      
      read(nu_ctl,*) h4rjb2rcd
      print *,' h4rjb2rcd = ', h4rjb2rcd
       
      format_out = ' '
      format_out = ""
      call trim_c(format_out, nc_format_out)
        
      write(nu_out, '(3x,a4, 4x,a3, 
     :                5x,a1, 5x,a3, 5x,a3, 5x,a3, 2x,a,    
     :                5x,a6, 6x,a5, 6x,a5, 
     :                7x,a4, 6x,a5, 6x,a5,
     :                1x,a, 1x,a, 1x,a)')  
     :               'freq', 'per', 
     :                'M',   'Rjb', 'Rcd', 'V30',  'Stress',   
     :                'y(cgs)', 'y/sig', 'y*sig', 
     :                'y(g)', 'y/sig', 'y*sig',
     :                'amp_lin', 'amp_nonlin', 'amp_total'      
* Loop over values of FMR, etc:

      call skipcmnt(nu_ctl,cmnts2skip,nc_cmnts2skip)      
! positioned to read fmrvs values

      readloop: DO

        buf = ' '
        read(nu_ctl,'(a)',IOSTAT=istatus) buf

        if (istatus /= 0) EXIT  ! could not read buf
        
        call trim_c(buf, nc_buf)
        
        buf4 = ' '
        buf4(1:4) = buf(1:4)
        call upstr(buf4)
        if (buf4 == 'STOP') EXIT

        READ(buf(1:nc_buf),*) freq, m, rjb, v30, stress
        IF (use_hard_rock_table(1:1) == 'Y') THEN
          v30 = 2500.0
        END IF
        
	rcd = sqrt(rjb**2 + h4rjb2rcd**2)
	
        call get_y_type(freq, y_type)

        SELECT CASE (y_type)        
        
        CASE ('PGVorPGA')

	  call ab06_fmrvs_evaluate_gmpes(freq, m, rcd, v30, stress, 
     :                        stress_adj_factor_delta, 
     :                        stress_adj_factor_ml, 
     :                        stress_adj_factor_mh,
     :                        saab06, sigab06, freq_tbl, coeff, nfreq,
     :                    amp_lin, amp_nonlin, amp_total)
          
        CASE ('PSA')
        
* Trap for freq outside the range of the tabulated frequencies (but only if not pga or pgv):

          call trap_freq_outside_range( freq, freq_tbl, nfreq, 
     :                                 indx_freqmax, itrapstatus)
     
          IF (itrapstatus /= 0) THEN
            write(*,*)
            write(*,*) ' freq = ', freq, ' is out of the range ', 
     :                  freq_tbl(1), ' to ', freq_tbl(indx_freqmax), ';'
            write(*,*) '; skipping to next entry in the control file.'
            CYCLE readloop
          END IF
             

          call get_freq_interval( freq, freq_tbl, nfreq, f1, f2)
          
!          write(*,*) ' For f = ', freq, ' f1, f2 = ', f1, f2

	  call ab06_fmrvs_evaluate_gmpes(f1, m, rcd, v30, stress,   
     :                        stress_adj_factor_delta, 
     :                        stress_adj_factor_ml, 
     :                        stress_adj_factor_mh,
     :                saab06_1, sigab06_1, freq_tbl, coeff, nfreq,
     :                    amp_lin1, amp_nonlin1, amp_total1)
     
	  call ab06_fmrvs_evaluate_gmpes(f2, m, rcd, v30, stress,  
     :                        stress_adj_factor_delta, 
     :                        stress_adj_factor_ml, 
     :                        stress_adj_factor_mh,
     :                saab06_2, sigab06_2, freq_tbl, coeff, nfreq,
     :                    amp_lin2, amp_nonlin2, amp_total2)
          

* Note: The interpolation is done in log space.

          x_interp = alog(freq)

          x1 = alog(f1)
          x2 = alog(f2)

          y1 = alog(saab06_1)
          y2 = alog(saab06_2)
          call interpolate(y1, y2, 
     :                     x1, x2, 
     :                     x_interp, alogy)
          saab06 = exp(alogy)
          
          y1 = alog(sigab06_1)
          y2 = alog(sigab06_2)
          call interpolate(y1, y2, 
     :                     x1, x2, 
     :                     x_interp, alogy)
          sigab06 = exp(alogy)
          
          y1 = alog(amp_lin1)
          y2 = alog(amp_lin2)
          call interpolate(y1, y2, 
     :                     x1, x2, 
     :                     x_interp, alogy)
          amp_lin = exp(alogy)
          
          y1 = alog(amp_nonlin1)
          y2 = alog(amp_nonlin2)
          call interpolate(y1, y2, 
     :                     x1, x2, 
     :                     x_interp, alogy)
          amp_nonlin = exp(alogy)
          
          y1 = alog(amp_total1)
          y2 = alog(amp_total2)
          call interpolate(y1, y2, 
     :                     x1, x2, 
     :                     x_interp, alogy)
          amp_total = exp(alogy)
          

        CASE DEFAULT
        
          PRINT *, ' Neither PGVorPGA or PSA were selected; '//
     :              'something is wrong; QUITTING'
          STOP

        END SELECT

        if(freq .lt. 0.0) then        
          scale_fact = 1.0
        else
          scale_fact = 1.0/981.0
        end if 
          
        write(nu_out,'(1x,f6.2,1x,f6.3,
     :                 1x,f5.2,1x,f7.2, 1x,f7.2, 1x,f7.1, 1x,f7.2,
     :                 6(1x,es10.3),
     :                 3x,f5.2, 6x,f5.2, 5x,f5.2)')
     :      freq, 1.0/freq,
     :      m, rjb, rcd, v30, stress,
     :                 saab06,            saab06/sigab06, 
     :                                    saab06*sigab06,
     :      scale_fact*saab06, scale_fact*saab06/sigab06, 
     :                         scale_fact*saab06*sigab06,
     :                 amp_lin, amp_nonlin, amp_total      
         
      END DO readloop ! loop back for another set of input parameters
      

      close(nu_ctl)
      close(nu_out)

      end
! ************************************  main ab06_gm_fmrvs

! ************************************  subroutine get_indices
      subroutine get_indices(freq_gmpe, nfreq_gmpe, 
     :             indx_freqmax, indx_pgv, indx_pga)
     
* Dates: 02/05/10 - Written by D. Boore, based on similar subroutine in 
*                   nga_gm_tmr, except changed "per" to "freq' and
*                   accounted for pga corresponding to freq = 99.0 rather
*                   than period = 0.0

! assume that the frequencies are sorted from small to large, and that frequency values
! for indices nfreq_gmpe-1 and nfreq_gmpe correspond to pga and pgv.

      real freq_gmpe(*)
      integer indx_freqmax, indx_pgv, indx_pga

! Find the index of the maximum frequency
      freq_max = -1.0
      DO i = 1, nfreq_gmpe-2
        IF (freq_gmpe(i) .gt. freq_max) THEN
          freq_max = freq_gmpe(i)
          indx_freqmax = i
        END IF
      END DO
 
!    Find index for pgv (frequency = -1.0):
      indx_pgv = 0
      find_pgv_index: DO i = 1, nfreq_gmpe
        IF (freq_gmpe(i) == -1.0) THEN
          indx_pgv = i
          EXIT find_pgv_index
        END IF
      END DO find_pgv_index
      IF (indx_pgv == 0) THEN
        print *,' ERROR; coefficients for pgv not found; QUIT'
        stop
      END IF
      
!    Find index for pga (frequency = 0.0):
      indx_pga = 0
      find_pga_index: DO i = 1, nfreq_gmpe
        IF (freq_gmpe(i) == 99.0) THEN
          indx_pga = i
          EXIT find_pga_index
        END IF
      END DO find_pga_index
      IF (indx_pga == 0) THEN
        PRINT *,' ERROR; coefficients for pga not found; QUIT'
        STOP
      END IF

      return
      end
! ************************************  subroutine get_indices
      
! ************************************  subroutine get_y_type
      subroutine get_y_type(freq, y_type)

* Dates: 02/05/10 - Written by D. Boore, based on similar subroutine in 
*                   nga_gm_tmr, except changed "per" to "freq'

      character y_type*(*)
      real freq
      
      y_type = ' '
      IF (freq == -1.0 .or. freq == 99.0) THEN
        y_type = 'PGVorPGA'
      ELSE IF (freq > 0.0 .and. freq < 99.0) THEN
        y_type = 'PSA' 
      ELSE
        write(*,*) ' ERROR, freq = ', freq, 
     :          ' NOT A VALID NUMBER; QUITTING!'
        STOP
      END IF
      
      return
      end
! ************************************  subroutine get_y_type
        
      
! ************************************  subroutine trap_freq_outside_range
      subroutine trap_freq_outside_range(freq, freq_gmpe, nfreq_gmpe, 
     :                                  indx_freqmax, itrapstatus)
     
* Dates: 02/05/10 - Written by D. Boore, based on similar subroutine in 
*                   nga_gm_tmr, except changed "per" to "freq'

      real freq, freq_gmpe(*)
      integer nfreq_gmpe, indx_freqmax
          
      if (freq < freq_gmpe(1) .or. freq > freq_gmpe(indx_freqmax) ) then
        print *,' CAUTION: freq = ', freq,
     :     ' is outside the tabulated range in freq_gmpe table'
        itrapstatus = -1
      else
        itrapstatus = 0
      end if
      
      return
      end       
! ************************************  subroutine trap_freq_outside_range

! ************************************  subroutine get_freq_interval
          subroutine get_freq_interval(freq, freq_gmpe, nfreq_gmpe,
     :                                   freq1, freq2)
     
* Dates: 02/05/10 - Written by D. Boore, based on similar subroutine in 
*                   nga_gm_tmr, except changed "per" to "freq'

      real freq, freq_gmpe(*), freq1, freq2
      integer nfreq_gmpe

* Find the two tabulated frequencies that span the interval
* containing the input frequency

! assume that the frequencies are sorted from small to large, and that frequency values
! for indices nfreq_gmpe-1 and nfreq_gmpe correspond to pga and pgv.

      if (freq == freq_gmpe(nfreq_gmpe-2)) then
        freq1 = freq_gmpe(nfreq_gmpe-3)
        freq2 = freq_gmpe(nfreq_gmpe-2)
        return
      else
        do i = 1, nfreq_gmpe - 3
          if (freq >= freq_gmpe(i) .and. freq < freq_gmpe(i+1) ) then
            freq1 = freq_gmpe(i)
            freq2 = freq_gmpe(i+1)
            return
          end if
        end do
      end if
      print *,' ERROR: could not find freq = ',freq,
     :      ' in freq_gmpe table'
      print *,' freq_gmpe '
      do i = 1, nfreq_gmpe
        print *, freq_gmpe(i)
      end do
      print *, ' QUITTING!!!'
      stop
  
      end
! ************************************  subroutine get_freq_interval
      
      include 'ab06_gm_fmrvs_utility_subs.for'

      include 'ab06_fmrvs_evaluate_gmpes.for'
        
!      include '\forprogs\get_lun.for'
!      include '\forprogs\skipcmnt.for'
!      include '\forprogs\skip.for'
!      include '\forprogs\upstr.for'
!      include '\forprogs\trim_c.for'
!      include '\forprogs\lin_interp.for'
!      include '\forprogs\locate.for'



